@@ -1,27 +1,13 @@
-/*=============================================================================
-
-    This file is part of ARB.
-
-    ARB is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    ARB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with ARB; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
+/*
     Copyright (C) 2012 Fredrik Johansson
 
-******************************************************************************/
+    This file is part of Arb.
+
+    Arb is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef ACB_POLY_H
 #define ACB_POLY_H
@@ -32,6 +18,7 @@
 #define ACB_POLY_INLINE static __inline__
 #endif
 
+#include <stdio.h>
 #include "acb.h"
 #include "arb_poly.h"
 
@@ -82,6 +69,27 @@ ACB_POLY_INLINE slong acb_poly_degree(const acb_poly_t poly)
     return poly->length - 1;
 }
 
+slong acb_poly_valuation(const acb_poly_t poly);
+
+ACB_POLY_INLINE int
+acb_poly_is_zero(const acb_poly_t z)
+{
+    return acb_poly_length(z) == 0;
+}
+
+ACB_POLY_INLINE int
+acb_poly_is_one(const acb_poly_t z)
+{
+    return (acb_poly_length(z) == 1) && acb_is_one(z->coeffs);
+}
+
+ACB_POLY_INLINE int
+acb_poly_is_x(const acb_poly_t z)
+{
+    return (acb_poly_length(z) == 2) && acb_is_zero(z->coeffs)
+        && acb_is_one(z->coeffs + 1);
+}
+
 ACB_POLY_INLINE void acb_poly_zero(acb_poly_t poly)
 {
     poly->length = 0;
@@ -129,7 +137,13 @@ void _acb_poly_majorant(arb_ptr res, acb_srcptr vec, slong len, slong prec);
 
 void acb_poly_majorant(arb_poly_t res, const acb_poly_t poly, slong prec);
 
-void acb_poly_printd(const acb_poly_t poly, slong digits);
+void acb_poly_fprintd(FILE * file, const acb_poly_t poly, slong digits);
+
+ACB_POLY_INLINE void
+acb_poly_printd(const acb_poly_t poly, slong digits)
+{
+    acb_poly_fprintd(stdout, poly, digits);
+}
 
 void _acb_poly_evaluate_horner(acb_t res, acb_srcptr f, slong len, const acb_t a, slong prec);
 void acb_poly_evaluate_horner(acb_t res, const acb_poly_t f, const acb_t a, slong prec);
@@ -157,10 +171,38 @@ void _acb_poly_integral(acb_ptr res, acb_srcptr poly, slong len, slong prec);
 
 void acb_poly_integral(acb_poly_t res, const acb_poly_t poly, slong prec);
 
+/* Transforms */
+
+void acb_poly_borel_transform(acb_poly_t res, const acb_poly_t poly, slong prec);
+
+void _acb_poly_borel_transform(acb_ptr res, acb_srcptr poly, slong len, slong prec);
+
+void acb_poly_inv_borel_transform(acb_poly_t res, const acb_poly_t poly, slong prec);
+
+void _acb_poly_inv_borel_transform(acb_ptr res, acb_srcptr poly, slong len, slong prec);
+
+void _acb_poly_binomial_transform_basecase(acb_ptr b, acb_srcptr a, slong alen, slong len, slong prec);
+
+void acb_poly_binomial_transform_basecase(acb_poly_t b, const acb_poly_t a, slong len, slong prec);
+
+void _acb_poly_binomial_transform_convolution(acb_ptr b, acb_srcptr a, slong alen, slong len, slong prec);
+
+void acb_poly_binomial_transform_convolution(acb_poly_t b, const acb_poly_t a, slong len, slong prec);
+
+void _acb_poly_binomial_transform(acb_ptr b, acb_srcptr a, slong alen, slong len, slong prec);
+
+void acb_poly_binomial_transform(acb_poly_t b, const acb_poly_t a, slong len, slong prec);
+
+
+
 void acb_poly_set(acb_poly_t dest, const acb_poly_t src);
 
 void acb_poly_set_round(acb_poly_t dest, const acb_poly_t src, slong prec);
 
+void acb_poly_set_trunc(acb_poly_t res, const acb_poly_t poly, slong n);
+
+void acb_poly_set_trunc_round(acb_poly_t res, const acb_poly_t poly, slong n, slong prec);
+
 void acb_poly_set_arb_poly(acb_poly_t poly, const arb_poly_t re);
 
 void acb_poly_set2_arb_poly(acb_poly_t poly, const arb_poly_t re, const arb_poly_t im);
@@ -220,6 +262,12 @@ void _acb_poly_sub(acb_ptr res, acb_srcptr poly1, slong len1,
 void acb_poly_sub(acb_poly_t res, const acb_poly_t poly1,
               const acb_poly_t poly2, slong prec);
 
+void acb_poly_add_series(acb_poly_t res, const acb_poly_t poly1,
+              const acb_poly_t poly2, slong len, slong prec);
+
+void acb_poly_sub_series(acb_poly_t res, const acb_poly_t poly1,
+              const acb_poly_t poly2, slong len, slong prec);
+
 ACB_POLY_INLINE void
 acb_poly_neg(acb_poly_t res, const acb_poly_t poly)
 {
@@ -236,6 +284,24 @@ acb_poly_scalar_mul_2exp_si(acb_poly_t res, const acb_poly_t poly, slong c)
     _acb_poly_set_length(res, poly->length);
 }
 
+ACB_POLY_INLINE void
+acb_poly_scalar_mul(acb_poly_t res, const acb_poly_t poly, const acb_t c, slong prec)
+{
+    acb_poly_fit_length(res, poly->length);
+    _acb_vec_scalar_mul(res->coeffs, poly->coeffs, poly->length, c, prec);
+    _acb_poly_set_length(res, poly->length);
+    _acb_poly_normalise(res);
+}
+
+ACB_POLY_INLINE void
+acb_poly_scalar_div(acb_poly_t res, const acb_poly_t poly, const acb_t c, slong prec)
+{
+    acb_poly_fit_length(res, poly->length);
+    _acb_vec_scalar_div(res->coeffs, poly->coeffs, poly->length, c, prec);
+    _acb_poly_set_length(res, poly->length);
+    _acb_poly_normalise(res);
+}
+
 void acb_poly_mullow_classical(acb_poly_t res, const acb_poly_t poly1,
                                             const acb_poly_t poly2,
                                                 slong n, slong prec);
@@ -315,6 +381,22 @@ void _acb_poly_div_root(acb_ptr Q, acb_t R, acb_srcptr A,
 
 /* Composition */
 
+void _acb_poly_taylor_shift_horner(acb_ptr poly, const acb_t c, slong n, slong prec);
+
+void acb_poly_taylor_shift_horner(acb_poly_t g, const acb_poly_t f, const acb_t c, slong prec);
+
+void _acb_poly_taylor_shift_divconquer(acb_ptr poly, const acb_t c, slong n, slong prec);
+
+void acb_poly_taylor_shift_divconquer(acb_poly_t g, const acb_poly_t f, const acb_t c, slong prec);
+
+void _acb_poly_taylor_shift_convolution(acb_ptr poly, const acb_t c, slong n, slong prec);
+
+void acb_poly_taylor_shift_convolution(acb_poly_t g, const acb_poly_t f, const acb_t c, slong prec);
+
+void _acb_poly_taylor_shift(acb_ptr poly, const acb_t c, slong n, slong prec);
+
+void acb_poly_taylor_shift(acb_poly_t g, const acb_poly_t f, const acb_t c, slong prec);
+
 void _acb_poly_compose(acb_ptr res,
     acb_srcptr poly1, slong len1,
     acb_srcptr poly2, slong len2, slong prec);
@@ -493,15 +575,25 @@ void _acb_poly_atan_series(acb_ptr res, acb_srcptr f, slong flen, slong n, slong
 
 void acb_poly_atan_series(acb_poly_t res, const acb_poly_t f, slong n, slong prec);
 
-void _acb_poly_exp_series_basecase(acb_ptr f,
-        acb_srcptr h, slong hlen, slong n, slong prec);
-
+void _acb_poly_exp_series_basecase(acb_ptr f, acb_srcptr h, slong hlen, slong n, slong prec);
 void acb_poly_exp_series_basecase(acb_poly_t f, const acb_poly_t h, slong n, slong prec);
-
 void _acb_poly_exp_series(acb_ptr f, acb_srcptr h, slong hlen, slong n, slong prec);
-
 void acb_poly_exp_series(acb_poly_t f, const acb_poly_t h, slong n, slong prec);
 
+void _acb_poly_sinh_cosh_series_basecase(acb_ptr s, acb_ptr c, const acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_sinh_cosh_series_basecase(acb_poly_t s, acb_poly_t c, const acb_poly_t h, slong n, slong prec);
+void _acb_poly_sinh_cosh_series_exponential(acb_ptr s, acb_ptr c, const acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_sinh_cosh_series_exponential(acb_poly_t s, acb_poly_t c, const acb_poly_t h, slong n, slong prec);
+void _acb_poly_sinh_cosh_series(acb_ptr s, acb_ptr c, const acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_sinh_cosh_series(acb_poly_t s, acb_poly_t c, const acb_poly_t h, slong n, slong prec);
+
+void _acb_poly_sinh_series(acb_ptr s, acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_sinh_series(acb_poly_t s, const acb_poly_t h, slong n, slong prec);
+
+void _acb_poly_cosh_series(acb_ptr c, acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_cosh_series(acb_poly_t c, const acb_poly_t h, slong n, slong prec);
+
+
 void _acb_poly_sin_cos_series_basecase(acb_ptr s,
                                     acb_ptr c, acb_srcptr h, slong hlen, slong n, slong prec, int times_pi);
 
@@ -550,6 +642,9 @@ void _acb_poly_tan_series(acb_ptr g, acb_srcptr h, slong hlen, slong len, slong
 
 void acb_poly_tan_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
 
+void _acb_poly_sinc_series(acb_ptr g, acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_sinc_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
+
 void _acb_poly_gamma_series(acb_ptr res, acb_srcptr h, slong hlen, slong len, slong prec);
 
 void acb_poly_gamma_series(acb_poly_t res, const acb_poly_t f, slong n, slong prec);
@@ -658,12 +753,14 @@ void acb_poly_elliptic_p_series(acb_poly_t res, const acb_poly_t z, const acb_t
 void _acb_poly_erf_series(acb_ptr g, acb_srcptr h, slong hlen, slong n, slong prec);
 void acb_poly_erf_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
 
-void _acb_poly_gamma_upper_series(acb_ptr g, const acb_t s, acb_srcptr h, slong hlen, slong n, slong prec);
-void acb_poly_gamma_upper_series(acb_poly_t g, const acb_t s, const acb_poly_t h, slong n, slong prec);
+ACB_POLY_INLINE slong
+acb_poly_allocated_bytes(const acb_poly_t x)
+{
+    return _acb_vec_allocated_bytes(x->coeffs, x->alloc);
+}
 
 #ifdef __cplusplus
 }
 #endif
 
 #endif
-
